@@ -16,7 +16,7 @@
 
 MAJOR_VERSION = "3"
 MINOR_VERSION = "10"
-PATCH_LEVEL = "3"
+PATCH_LEVEL = "4"
 VERSION_STRING = '%s.%s.%s' % (MAJOR_VERSION, MINOR_VERSION, PATCH_LEVEL)
-HAVE_LINUX_AWE_VOICE_H = "yes" == "yes"
+HAVE_LINUX_AWE_VOICE_H = "" == "yes"
 ENABLE_TUNER = "no" == "yes"
