@@ -433,7 +433,12 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         self.g_synth = gtk.combo_box_new_text()
         if winmidi:
             for devname in winmidi.output_devices():
-                self.g_synth.append_text(devname)
+                #FIXME workaround of the bug
+                # http://code.google.com/p/solfege/issues/detail?id=12
+                if devname is None:
+                    self.g_synth.append_text("FIXME bug #12")
+                else:
+                    self.g_synth.append_text(devname)
         self.g_synth.set_active(self.get_int('sound/synth_number') + 1)
         hbox.pack_start(self.g_device_radio, False)
         hbox.pack_start(self.g_synth, False)
