@@ -43,8 +43,8 @@ class Menu(dict):
         self['children'] = children
     children = property(get_children, set_children)
     def pformat(self, indent):
-        return "%(indent)s{'name': '%(name)s',\n" \
-        "%(indent)s 'nametotranslate': _('%(name)s'),\n" \
+        return "%(indent)s{'name': %(name)r,\n" \
+        "%(indent)s 'nametotranslate': _(%(name)r),\n" \
         "%(indent)s 'children': [\n" \
         "%(children)s ]\n" \
         "%(indent)s}," %  {
@@ -56,7 +56,7 @@ class Menu(dict):
         v = []
         for child in self.children:
             if isinstance(child, basestring):
-                v.append("%s'%s'," % (" " * indent, child))
+                v.append("%s%r," % (" " * indent, child))
             else:
                 v.append(child.pformat(indent))
         return "\n".join(v)
