@@ -252,7 +252,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.m_app = app.SolfegeApp(options, self, lessonfile_manager)
         self.g_ui_manager = gtk.UIManager()
         self.m_action_groups = {
-            'Practise': gtk.ActionGroup('Practise'),
+            'Exercises': gtk.ActionGroup('Exercises'),
             'Exit': gtk.ActionGroup('Exit'),
             'NotExit': gtk.ActionGroup('NotExit'),
         }
@@ -397,46 +397,63 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         This function has to be called after we have cleared out any
         lesson_id crashes.
         """
+        def escape(s):
+            return s.replace('&','&amp;').replace("'", "&apos;").replace('"', '&quot;')
         actions = []
         if self.m_practise_and_test_merge_id:
             self.g_ui_manager.remove_ui(self.m_practise_and_test_merge_id)
             self.g_ui_manager.ensure_update()
+        # This method can be called more than once. So we must first
+        # remove any actions from the last time it was called.
+        for action in self.m_action_groups['Exercises'].list_actions():
+            self.m_action_groups['Exercises'].remove_action(action)
+        # We use this counter to make sure all Action names are unique.
+        # This to avoid warnings if a toplevel menu and a submenu has
+        # the same name
+        id_counter = 0
         s = "<menubar name='Menubar'>"
         tests = "<menu action='TestsMenu'>"
         for menu in self.m_tree.m_menus:
-            self.m_action_groups['NotExit'].add_action(
-                gtk.Action(menu.name, _(menu.name), None, None))
-            s += "<menu action='%s'>" % menu.name
+            id_counter += 1
+            self.m_action_groups['Exercises'].add_action(
+                gtk.Action("%s%i" % (menu.name, id_counter),
+                           _(menu.name), None, None))
+            s += "<menu action='%s%i'>" % (escape(menu.name), id_counter)
             # The following line might look strange here, but it makes sure
             # that the menus on the menu bar stay at the correct order after
             # we save the learning tree in the learning tree editor.
-            self.g_ui_manager.add_ui_from_string("<menubar name='Menubar'><menu action='%s'/></menubar>" % menu.name)
+            self.g_ui_manager.add_ui_from_string("<menubar name='Menubar'><menu action='%s%i'/></menubar>" % (escape(menu.name), id_counter))
             for topic in menu.children:
-                s += "<menu action='%s'>" % topic['name']
-                tests += "<menu action='TEST_%s'>" % topic['name']
-                actions.append((topic['name'], None, _(topic['name'])))
-                actions.append(('TEST_%s' % topic['name'], None, _(topic['name'])))
+                id_counter += 1
+                s += "<menu action='%s%i'>" % (escape(topic['name']), id_counter)
+                tests += "<menu action='TEST_%s%i'>" % (escape(topic['name']), id_counter)
+                actions.append(("%s%i" % (topic['name'], id_counter),
+                                None, _(topic['name'])))
+                actions.append(('TEST_%s%i' % (topic['name'], id_counter),
+                               None, _(topic['name'])))
                 for lesson_id in topic.children:
                     if self.m_tree.m_visibilities[lesson_id] > self.m_tree.m_visibility:
                         continue
                     menutitle = self.m_app.lessonfile_manager.m_uiddb[lesson_id]['header']['title']
+                    id_counter += 1
                     actions.append((
-                        lesson_id, None, menutitle, None, None,
+                        "%s%i" % (lesson_id, id_counter),
+                        None, menutitle, None, None,
                         lambda o, e=lesson_id: self.m_app.practise_lesson_id(e)))
                     if self.m_app.lessonfile_manager.is_test_passed(lesson_id):
                         menutitle = _("%s (passed)") % menutitle
                     if self.m_app.lessonfile_manager.get(lesson_id, 'test'):
                         actions.append((
-                            'TEST_%s' % lesson_id, None,
+                            'TEST_%s%s' % (lesson_id, id_counter), None,
                             menutitle,
                             None, None,
                             lambda o, e=lesson_id: self.m_app.test_lesson_id(e)))
-                        tests += "<menuitem action='TEST_%s' />" % lesson_id
-                    s += "<menuitem action='%s' />" % lesson_id
+                        tests += "<menuitem action='TEST_%s%i' />" % (escape(lesson_id), id_counter)
+                    s += "<menuitem action='%s%i' />" % (escape(lesson_id), id_counter)
                 s += "</menu>"
                 tests += "</menu>"
             s += "</menu>"
-        self.m_action_groups['NotExit'].add_actions(actions)
+        self.m_action_groups['Exercises'].add_actions(actions)
         tests += "</menu>"
         s += tests
         s += "</menubar>"
@@ -798,6 +815,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             gu.dialog_ok(_("The '%s' exercise module does not support test yet." % self.m_viewer))
             return
         self.m_action_groups['NotExit'].set_sensitive(False)
+        self.m_action_groups['Exercises'].set_sensitive(False)
         self.g = self.box_dict[self.m_viewer].g_notebook.get_nth_page(0)
         self.box_dict[self.m_viewer].g_notebook.get_nth_page(0).reparent(self.main_box)
         self.box_dict[self.m_viewer].g_notebook.hide()
@@ -805,6 +823,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def exit_test_mode(self):
         self.m_app.m_test_mode = False
         self.m_action_groups['NotExit'].set_sensitive(True)
+        self.m_action_groups['Exercises'].set_sensitive(True)
         box = gtk.VBox()
         self.box_dict[self.m_viewer].g_notebook.insert_page(box, gtk.Label(_("Practise")), 0)
         self.g.reparent(box)
