@@ -1,5 +1,5 @@
 # Solfege - free ear training software
-# Copyright (C) 2007 Tom Cato Amundsen
+# Copyright (C) 2007, 2008 Tom Cato Amundsen
 # License is GPL, see file COPYING
 
 import unittest
@@ -12,6 +12,7 @@ from src.lessonfile import *
 from src import dataparser
 import mpd
 from src import cfg
+from mpd import mpdutils
 
 class TestParser(unittest.TestCase):
     def test_bad_musictype(self):
@@ -129,10 +130,10 @@ class TestVoice(unittest.TestCase):
         self.p._idx = 0
     def test_get_mpd_music_string(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
-            r"\staff{ c d e }")
+            "\\staff{\nc d e\n}")
         self.p.header.random_transpose = True
         self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
-            r"\staff\transpose d'{ c d e }")
+            "\\staff\\transpose d'{\nc d e\n}")
     def test_get_lilypond_code(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code(self.p), r"\transpose c' d'{ c d e }")
     def test_get_lilypond_code_first_note(self):
@@ -147,6 +148,108 @@ class TestVoice(unittest.TestCase):
         question['music'].play_slowly(self.p, question)
         self.assertEquals(soundcard.synth.flush_testdata(),
             "t30/4 p0:0 n48 d1/4 o48 n50 d1/4 o50 n52 d1/4 o52")
+    def test_findfirst(self):
+        for p1, p2, s, n in (
+            (0, 1, "c d e", "c"),
+            (1, 4, "<c'' e>", "c''"),
+            (2, 5, "< c'' e>", "c''"),
+            (2, 5, "\n\nERR", "ERR"),
+            (1, 4, " c,,16", "c,,"),
+            (3, 4, " \n c d e", "c"),
+            (11, 12, r"\clef bass c d e", "c"),
+            (23, 26, r"\clef violin \time 3/8 fis", "fis"),
+            (17, 18, r'\clef "violin_8" d', "d"),
+            (25, 27, "\\key g \\major \\time 2/4\n d'8 | [g g]", "d'"),
+            (1, 2, "\nc", "c"),
+            ):
+            x = Rvoice(s)
+            self.assertEquals((p1, p2), mpdutils.find_possible_first_note(x.m_musicdata), "String failed: %s: %s" % (s, mpdutils.find_possible_first_note(x.m_musicdata)))
+            self.assertEquals(s[p1:p2], n)
+
+class TestErrorHandling(unittest.TestCase):
+    def setUp(self):
+        cfg.set_bool('config/override_default_instrument', False)
+    def _dotest(self, musictypes, bad_code):
+        for t in musictypes:
+            for code, err_context in bad_code:
+                self.p = QuestionsLessonfile()
+                self.p.parse_string('''
+                header { random_transpose = no }
+                question { music = %s("%s") }
+                ''' % (t, code))
+                self.p.m_transpose = mpd.MusicalPitch.new_from_notename("d'")
+                self.p._idx = 0
+                question = self.p.m_questions[0]
+                try:
+                    self.p.play_question(question)
+                except MusicObjectException, e:
+                    self.assertEquals(err_context, str(e))
+                except mpd.MpdException, e:
+                    ec = question.music.get_err_context(e, self.p)
+                    self.assertEquals(err_context, ec, "Music failed with musictype %s:\n%s !=\n%s" % (t, err_context, ec))
+                else:
+                    assert False, "No error found in code %s" % code
+    def test_1(self):
+        self._dotest(("chord", "voice", "rvoice", "rhythm", "percussion", "rhythm"),
+                   (("c d e ERR f",
+                     "c d e ERR f\n"
+                     "      ^^^"),
+                    ("c d e f\ng ERR a",
+                    "g ERR a\n"
+                     "  ^^^"),
+                    ("c d XX e f\ng ERR a",
+                     "c d XX e f\n"
+                     "    ^^"),
+                    ("ERR",
+                     "ERR\n"
+                     "^^^"),
+                    ("4ERR",
+                     "4ERR\n"
+                     "^"),
+                  ))
+    def test_satb(self):
+        self._dotest(("satb",),
+            (("c'' | ERR | g | c",
+              "c'' | ERR | g | c\n"
+              "     ^^^^^"),
+             ("c''|g'|ERR| c",
+              "c''|g'|ERR| c\n"
+              "       ^^^"),
+             ("c'' |e' d'|\n ERR |c",
+              "The music code from the lesson file has been modified by\n"
+              "removing the new-line characters. This to more easily show\n"
+              "where the error occured. Satb music should not contain music\n"
+              "characters.\n"
+              "c'' |e' d'| ERR |c\n"
+              "           ^^^^^"),
+             ("|||",
+              "Satb music does not allow an empty voice"),
+             ("{ | g",
+              "Satb music should be divided into 4 parts by the '|' character",),
+              ))
+    def test_music(self):
+        self._dotest(("music",),
+            (("\\staff{ c' d ERR e }",
+              "\\staff{ c' d ERR e }\n"
+              "             ^^^"),
+             ("\\staff{ c4. c. }",
+              "\\staff{ c4. c. }\n"
+              "            ^^"),
+             ("\\addvoice{ jojo ",
+              "\\addvoice{ jojo \n"
+              "^^^^^^^^^"),
+             ("\\staff { c4 c4. c. }",
+              "\\staff { c4 c4. c. }\n"
+              "                ^^"),
+             ("\\staff { c4\n"
+              "ERR }",
+              "ERR }\n"
+              "^^^"),
+             ("\\staff{ \\clef ERROR c",
+              "\\staff{ \\clef ERROR c\n"
+                "        ^^^^^^^^^^^"),
+            ))
+
 
 class TestRvoice(unittest.TestCase):
     def setUp(self):
@@ -155,15 +258,16 @@ class TestRvoice(unittest.TestCase):
         self.p.parse_string("""
         header { random_transpose = no }
         question { music = rvoice("c' d b c") }
+        question { music = rvoice("[c8' d] e16") }
         """)
         self.p.m_transpose = mpd.MusicalPitch.new_from_notename("d'")
         self.p._idx = 0
     def test_get_mpd_music_string(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
-            r"\staff\relative c'{ c d b c }")
+            "\\staff\\relative c'{\nc d b c\n}")
         self.p.header.random_transpose = True
         self.assertEquals(self.p.m_questions[0]['music'].get_mpd_music_string(self.p),
-            r"\staff\transpose d'\relative c'{ c d b c }")
+            "\\staff\\transpose d'\\relative c'{\nc d b c\n}")
     def test_play(self):
         question = self.p.m_questions[0]
         question['music'].play(self.p, question)
@@ -178,6 +282,8 @@ class TestRvoice(unittest.TestCase):
         self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code(self.p), r"\transpose c' d'\relative c{ c' d b c }")
     def test_get_lilypond_code_first_note(self):
         self.assertEquals(self.p.m_questions[0]['music'].get_lilypond_code_first_note(self.p), r"\score{ \new Staff<<   \new Voice\transpose c' d'\relative c{ \cadenzaOn c' }  \new Voice{ \hideNotes c' d b c }  >>  \layout { ragged-last = ##t } }")
+    def test_with_bream_first_note(self):
+        self.assertEquals(self.p.m_questions[1].music.get_mpd_music_string(self.p), "\\staff\\relative c{\n[c8' d] e16\n}")
 
 class TestSatb(unittest.TestCase):
     def setUp(self):
@@ -265,6 +371,9 @@ class TestWavfile(unittest.TestCase):
         question = self.p.m_questions[0]
         self.assertEquals(question['music'].get_mpd_music_string(self.p),
                           "Wavfile:lesson-files/share/fifth-small-293.33.wav")
+        self.assertEquals(question.music.m_musicdata,
+                          "lesson-files/share/fifth-small-293.33.wav")
+        self.assertTrue(isinstance(question.music, Wavfile))
 
 class TestMidifile(unittest.TestCase):
     def setUp(self):
@@ -282,6 +391,9 @@ class TestMidifile(unittest.TestCase):
         question = self.p.m_questions[0]
         self.assertEquals(question['music'].get_mpd_music_string(self.p),
                           "Midifile:lesson-files/share/fanfare.midi")
+        self.assertEquals(question.music.m_musicdata,
+                          "lesson-files/share/fanfare.midi")
+        self.assertTrue(isinstance(question.music, Midifile))
 
 
 class TestCmdline(unittest.TestCase):
@@ -357,9 +469,47 @@ class TestLessonfileMisc(unittest.TestCase):
         self.assertNotEquals(c, d)
         self.assertNotEquals(d, e)
         self.assertNotEquals(d, f)
+    def test_IdPropertyLessonfileFilter(self):
+        s = """header {
+                qprops = "name", "inversion"
+                qprop_labels = _("Name"), _("Inversion")
+            }
+            question { name = "one" }
+            question { name = "two" inversion=2 }
+            """
+        p = IdPropertyLessonfile()
+        p.parse_string(s)
+        # One question should be discarded because it does not have
+        # the inversion variable
+        self.assertEquals(len(p.m_discards), 1)
+        self.assertEquals(len(p.m_questions), 1)
+    def test_IdPropertyLessonfileFilter2(self):
+        s = 'header { qprops = "name", "inversion" qprop_labels = "Name", "Inversion" }\n' \
+            'question { name = "one" inversion=1} \n' \
+            'question { name = "two" inversion=2 top=0 } '
+        p = IdPropertyLessonfile()
+        p.parse_string(s)
+        # No questions are discaded.
+        # top is not a property since it is not added to qprops, so
+        # thats why the first question is not discarded.
+        self.assertEquals(len(p.m_discards), 0)
+        self.assertEquals(len(p.m_questions), 2)
+    def test_IdPropertyLessonfile_qprops(self):
+        s = 'header { qprops = "name", "inversion", "toptone" qprop_labels = "Name", "Inversion", "Toptone" }\n' \
+            'question { name = "one" inversion=1} \n' \
+            'question { name = "two" inversion=2 top=0 } '
+        p = IdPropertyLessonfile()
+        p.parse_string(s)
+        # No questions are discarded, since they both contain the same
+        # two properties, of the tree listed in qprops. The 'top' variable
+        # in the second question does not affect what is or is not discarded
+        # because is it not a property (not defined in qprops)
+        self.assertEquals(len(p.m_questions), 2)
     def test_ChordLessonfileFilter(self):
-        s = 'question { name = "one" } \n' \
-            'question { name = "two" inversion=2 } '
+        s = """header { }
+            question { name = "one" }
+            question { name = "two" inversion=2 }
+            """
         p = ChordLessonfile()
         p.parse_string(s)
         # One question should be discarded because it does not have
@@ -375,6 +525,15 @@ class TestLessonfileMisc(unittest.TestCase):
         # top is not a property since it is not added to qprops, so
         # thats why the first question is not discarded.
         self.assertEquals(len(p.m_questions), 2)
+    def test_IdProperty(self):
+        """
+        Once, a single item qprops did not work.
+        """
+        s = 'header { qprops = "name" qprop_labels = "NAME" } ' \
+            'question { name = "major" music = chord("c e g") } '
+        p = IdPropertyLessonfile()
+        p.parse_string(s)
+        self.assertEquals(p.header.qprops, [u'name'])
 
 
 suite = unittest.makeSuite(TestParser)
@@ -391,3 +550,4 @@ suite.addTest(unittest.makeSuite(TestCmdline))
 suite.addTest(unittest.makeSuite(TestLessonfileHeader))
 suite.addTest(unittest.makeSuite(TestLessonfileQuestion))
 suite.addTest(unittest.makeSuite(TestLessonfileMisc))
+suite.addTest(unittest.makeSuite(TestErrorHandling))
